#include "controller.h"
#include "delays.h"
#include "types.h"
#include "globals.h"


static enum State {
	FLOOR0TOSTOP, FLOOR0STOP, FLOOR0TOOPEN, FLOOR0OPEN, FLOOR0TOCLOSE,
	FLOOR1TOSTOP, FLOOR1STOP, FLOOR1TOOPEN, FLOOR1OPEN, FLOOR1TOCLOSE,
	FLOOR2TOSTOP, FLOOR2STOP, FLOOR2TOOPEN, FLOOR2OPEN, FLOOR2TOCLOSE,
	FLOOR3TOSTOP, FLOOR3STOP, FLOOR3TOOPEN, FLOOR3OPEN, FLOOR3TOCLOSE,
	UPFROM0, FLOOR0TO1, DOWNTO1,
	UPFROM1, FLOOR1TO2, DOWNTO2,
	UPFROM2, FLOOR2TO3, DOWNTO3TOSTOP,
	DOWNFROM3, FLOOR3TO2, UPTO2,
	DOWNFROM2, FLOOR2TO1, UPTO1,
	DOWNFROM1, FLOOR1TO0, UPTO0
};
static enum Signal {
	ABSENT, OPEN, LIGHTOPEN, CLOSEDOOR, OPENDOOR,
	STOPTOUP, UPFLOOR, FLOORDOWN, FLOORSTOP, UPDIR,
	STOP, STOPTODOWN, DOWNFLOOR, FLOORUP, UPTOSTOP, DOWNDIR 
};

static myclock ctrlclk;

void ControllerInit(Controller *me){
	me->state_ = FLOOR0STOP;
	open = 0;
	dir = 1; //DIRECTIONUP
	halt = 0;
	resetClk(&ctrlclk);
}
void ControllerTran_(Controller *me, unsigned int dest)
{
	me->state_ = dest;
}
void ControllerDisplay (Controller *me ){

/*	EE_lcd_goto ( 14, 0 );
    EE_lcd_putc( me->state_ + 48 );
/*		EE_lcd_goto ( 7, 0 );
		EE_lcd_putc(' ');
		break;	
*/
}

enum Signal ControllerGetSignals(Controller *me){
	switch ( me->state_ ){
//floor 0 
		case FLOOR0TOSTOP:
			if ( ishalt )
				return STOP;
		case FLOOR0STOP:
			if (b0)
				return OPEN;
			else if ( b1 || b2 || b3 )
				return STOPTOUP;
		case FLOOR0TOOPEN:
			if ( isopen && !lock0 )
				return LIGHTOPEN;
		case FLOOR0OPEN:
			if (ExpAlarm(&ctrlclk))
				return CLOSEDOOR;
		case FLOOR0TOCLOSE:
			if ( !isopen && lock0 )
				return OPENDOOR;
		case UPFROM0:
			if ( !is0 )
				return UPFLOOR;
		case FLOOR0TO1:
			if ( is1 )
				return FLOORDOWN;
		case DOWNTO1:
			if ( b1 )
				return FLOORSTOP;
			else
				return UPDIR;
		case FLOOR1TOSTOP:
			if ( ishalt )
				return STOP;
//floor 1 
		case FLOOR1STOP:
			if (b1)
				return OPEN;
			else if ( (dir || !b0) && (b2 || b3) )
				return STOPTOUP;
			else if ( (!dir || (!b2 && !b3)) && b0 )
				return STOPTODOWN;
		case FLOOR1TOOPEN:
			if ( isopen && !lock1 )
				return LIGHTOPEN;
		case FLOOR1OPEN:
			if (ExpAlarm(&ctrlclk))
				return CLOSEDOOR;
		case FLOOR1TOCLOSE:
			if ( !isopen && lock1 )
				return OPENDOOR;
		case UPFROM1:
			if ( !is1 )
				return UPFLOOR;
		case FLOOR1TO2:
			if ( is2 )
				return FLOORDOWN;
		case DOWNTO2:
			if ( b2 )
				return FLOORSTOP;
			else
				return UPDIR;
		case FLOOR2TOSTOP:
			if ( ishalt )
				return STOP;
		case DOWNFROM1:
			if ( !is1 )
				return DOWNFLOOR;
		case FLOOR1TO0:
			if ( is0 )
				return FLOORUP;
		case UPTO0:
				return UPTOSTOP;
//floor 2 
		case FLOOR2STOP:
			if (b2)
				return OPEN;
			else if ( (dir || (!b0 && !b1)) &&  b3 )
				return STOPTOUP;
			else if ( (!dir || !b3) && (b0 || b1 ))
				return STOPTODOWN;
		case FLOOR2TOOPEN:
			if ( isopen && !lock2 )
				return LIGHTOPEN;
		case FLOOR2OPEN:
			if (ExpAlarm(&ctrlclk))
				return CLOSEDOOR;
		case FLOOR2TOCLOSE:
			if ( !isopen && lock2 )
				return OPENDOOR;
		case UPFROM2:
			if ( !is2 )
				return UPFLOOR;
		case FLOOR2TO3:
			if ( is3 )
				return FLOORSTOP;
		case DOWNTO3TOSTOP:
			if ( ishalt )
				return STOP;
		case DOWNFROM2:
			if ( !is2 )
				return DOWNFLOOR;
		case FLOOR2TO1:
			if ( is1 )
				return FLOORUP;
		case UPTO1:
			if ( b1 )
				return UPTOSTOP;
			else
				return DOWNDIR;
//floor 3 
		case FLOOR3STOP:
			if (b3)
				return OPEN;
			else if ( b0 || b1 ||  b2 )
				return STOPTODOWN;
		case FLOOR3TOOPEN:
			if ( isopen && !lock3 )
				return LIGHTOPEN;
		case FLOOR3OPEN:
			if (ExpAlarm(&ctrlclk))
				return CLOSEDOOR;
		case FLOOR3TOCLOSE:
			if ( !isopen && lock3 )
				return OPENDOOR;
		case DOWNFROM3:
			if ( !is3 )
				return DOWNFLOOR;
		case FLOOR3TO2:
			if ( is2 )
				return FLOORUP;
		case UPTO2:
			if ( b2 )
				return UPTOSTOP;
			else
				return DOWNDIR;

	}
	return ABSENT;
}
void ControllerDispatch(Controller *me){

	enum Signal s = ControllerGetSignals(me);
	switch ( me->state_ ){
//FLOOR 0
		case FLOOR0TOSTOP:
			switch ( s ) {
				case STOP:
					halt = 0;
					ControllerTran_ ( me, FLOOR0STOP );
					break;
			}
			break;
		case FLOOR0STOP:
			switch ( s ) {
				case OPEN:
					open = 1;
					ControllerTran_ ( me, FLOOR0TOOPEN );
					break;
				case STOPTOUP:
					up = dir = 1;
					ControllerTran_ ( me, UPFROM0 );
					break;
			}
			break;	
		case FLOOR0TOOPEN: 
			switch ( s ) {
				case LIGHTOPEN:
					light0 = open = 0;
					SetAlarm(&ctrlclk, 0, DELAYOC);
					ControllerTran_ ( me, FLOOR0OPEN );
					break;
			}
			break;
		case FLOOR0OPEN:
			switch ( s ) {
				case CLOSEDOOR:
					close = 1;
					ControllerTran_ ( me, FLOOR0TOCLOSE );
					break;
			}
			break;
		case FLOOR0TOCLOSE:
			switch ( s ) {
				case OPENDOOR:
					close = 0;
					ControllerTran_ ( me, FLOOR0STOP );
					break;
			}
			break;
		case UPFROM0:
			switch ( s ) {
				case UPFLOOR:
					ControllerTran_ ( me, FLOOR0TO1 );
					break;
			}
			break;
		case FLOOR0TO1:
			switch ( s ) {
				case FLOORDOWN:
					ControllerTran_ ( me, DOWNTO1 );
					break;
			}
			break;
		case DOWNTO1:
			switch ( s ) {
				case FLOORSTOP:
					halt = 1;
					up = 0;
					ControllerTran_ ( me, FLOOR1TOSTOP );
					break;
				case UPDIR:
					ControllerTran_ ( me, UPFROM1 );
					break;
			}
			break;
		case FLOOR1TOSTOP:
			switch ( s ) {
				case STOP:
					halt = 0;
					ControllerTran_ ( me, FLOOR1STOP );
					break;
			}
			break;
//FLOOR 1
		case FLOOR1STOP:
			switch ( s ) {
				case OPEN:
					open = 1;
					ControllerTran_ ( me, FLOOR1TOOPEN );
					break;
				case STOPTOUP:
					up = dir = 1;
					ControllerTran_ ( me, UPFROM1 );
					break;
				case STOPTODOWN:
					dir = 0;
					down = 1;
					ControllerTran_ ( me, DOWNFROM1 );
					break;
			}
			break;	
		case FLOOR1TOOPEN: 
			switch ( s ) {
				case LIGHTOPEN:
					light1 = open = 0;
					SetAlarm(&ctrlclk, 0, DELAYOC);
					ControllerTran_ ( me, FLOOR1OPEN );
					break;
			}
			break;
		case FLOOR1OPEN:
			switch ( s ) {
				case CLOSEDOOR:
					close = 1;
					ControllerTran_ ( me, FLOOR1TOCLOSE );
					break;
			}
			break;
		case FLOOR1TOCLOSE:
			switch ( s ) {
				case OPENDOOR:
					close = 0;
					ControllerTran_ ( me, FLOOR1STOP );
					break;
			}
			break;
		case UPFROM1:
			switch ( s ) {
				case UPFLOOR:
					ControllerTran_ ( me, FLOOR1TO2 );
					break;
			}
			break;
		case FLOOR1TO2:
			switch ( s ) {
				case FLOORDOWN:
					ControllerTran_ ( me, DOWNTO2 );
					break;
			}
			break;
		case DOWNTO2:
			switch ( s ) {
				case FLOORSTOP:
					halt = 1;
					up = 0;
					ControllerTran_ ( me, FLOOR2TOSTOP );
					break;
				case UPDIR:
					ControllerTran_ ( me, UPFROM2 );
					break;
			}
			break;
		case FLOOR2TOSTOP:
			switch ( s ) {
				case STOP:
					halt = 0;
					ControllerTran_ ( me, FLOOR2STOP );
					break;
			}
			break;
		case DOWNFROM1:
			switch ( s ) {
				case DOWNFLOOR:
					ControllerTran_ ( me, FLOOR1TO0 );
					break;
			}
			break;
		case FLOOR1TO0:
			switch ( s ) {
				case FLOORUP:
					ControllerTran_ ( me, UPTO0 );
					break;
			}
			break;
		case UPTO0:
			switch ( s ) {
				case UPTOSTOP:
					halt=1;
					down=0;
					ControllerTran_ ( me, FLOOR0TOSTOP );
					break;
			}
			break;
//FLOOR 2
		case FLOOR2STOP:
			switch ( s ) {
				case OPEN:
					open = 1;
					ControllerTran_ ( me, FLOOR2TOOPEN );
					break;
				case STOPTOUP:
					up = dir = 1;
					ControllerTran_ ( me, UPFROM2 );
					break;
				case STOPTODOWN:
					dir = 0;
					down = 1;
					ControllerTran_ ( me, DOWNFROM2 );
					break;
			}
			break;	
		case FLOOR2TOOPEN: 
			switch ( s ) {
				case LIGHTOPEN:
					light2 = open = 0;
					SetAlarm(&ctrlclk, 0, DELAYOC);
					ControllerTran_ ( me, FLOOR2OPEN );
					break;
			}
			break;
		case FLOOR2OPEN:
			switch ( s ) {
				case CLOSEDOOR:
					close = 1;
					ControllerTran_ ( me, FLOOR2TOCLOSE );
					break;
			}
			break;
		case FLOOR2TOCLOSE:
			switch ( s ) {
				case OPENDOOR:
					close = 0;
					ControllerTran_ ( me, FLOOR2STOP );
					break;
			}
			break;
		case UPFROM2:
			switch ( s ) {
				case UPFLOOR:
					ControllerTran_ ( me, FLOOR2TO3 );
					break;
			}
			break;
		case FLOOR2TO3:
			switch ( s ) {
				case FLOORSTOP:
					halt = 1;
					up = 0;
					ControllerTran_ ( me, DOWNTO3TOSTOP );
					break;
			}
			break;
		case DOWNTO3TOSTOP:
			switch ( s ) {
				case STOP:
					halt = 0;
					ControllerTran_ ( me, FLOOR3STOP );
					break;
			}
			break;
		case DOWNFROM2:
			switch ( s ) {
				case DOWNFLOOR:
					ControllerTran_ ( me, FLOOR2TO1 );
					break;
			}
			break;
		case FLOOR2TO1:
			switch ( s ) {
				case FLOORUP:
					ControllerTran_ ( me, UPTO1 );
					break;
			}
			break;
		case UPTO1:
			switch ( s ) {
				case UPTOSTOP:
					halt=1;
					down=0;
					ControllerTran_ ( me, FLOOR1TOSTOP );
					break;
				case DOWNDIR:
					ControllerTran_ ( me, DOWNFROM1 );
					break;
			}
			break;
//FLOOR 3
		case FLOOR3STOP:
			switch ( s ) {
				case OPEN:
					open = 1;
					ControllerTran_ ( me, FLOOR3TOOPEN );
					break;
				case STOPTODOWN:
					dir = 0;
					down = 1;
					ControllerTran_ ( me, DOWNFROM3 );
					break;
			}
			break;	
		case FLOOR3TOOPEN: 
			switch ( s ) {
				case LIGHTOPEN:
					light3 = open = 0;
					SetAlarm(&ctrlclk, 0, DELAYOC);
					ControllerTran_ ( me, FLOOR3OPEN );
					break;
			}
			break;
		case FLOOR3OPEN:
			switch ( s ) {
				case CLOSEDOOR:
					close = 1;
					ControllerTran_ ( me, FLOOR3TOCLOSE );
					break;
			}
			break;
		case FLOOR3TOCLOSE:
			switch ( s ) {
				case OPENDOOR:
					close = 0;
					ControllerTran_ ( me, FLOOR3STOP );
					break;
			}
			break;

		case DOWNFROM3:
			switch ( s ) {
				case DOWNFLOOR:
					ControllerTran_ ( me, FLOOR3TO2 );
					break;
			}
			break;
		case FLOOR3TO2:
			switch ( s ) {
				case FLOORUP:
					ControllerTran_ ( me, UPTO2 );
					break;
			}
			break;
		case UPTO2:
			switch ( s ) {
				case UPTOSTOP:
					halt=1;
					down=0;
					ControllerTran_ ( me, FLOOR2TOSTOP );
					break;
				case DOWNDIR:
					ControllerTran_ ( me, DOWNFROM2 );
					break;
			}
			break;
	}
	ControllerDisplay ( me );

}



					

