/*
 * Authors : Marco di Natale and Paolo Pagano
 */
#define NUMBUTTONS 4
#include "p33FJ256MC710.h"
#include "lcd-protocol.h"
#include "pbl.h"
#include "cabin.h"
#include "controller.h"
#include "motorshaft.h"
#include "stdio.h"
#include "delays.h"
#include "globals.h"


PBL pbl0, pbl1, pbl2, pbl3;
Cabin cbn;
Controller cnt;
Motor mtr;
unsigned int dummyState;

// Primary (XT, HS, EC) Oscillator without PLL
_FOSCSEL(FNOSC_PRI);
// OSC2 Pin Function: OSC2 is Clock Output - Primary Oscillator Mode: XT Crystanl
_FOSC(OSCIOFNC_ON & POSCMD_XT);
// Watchdog Timer Enabled/disabled by user software
_FWDT(FWDTEN_OFF);
// Disable Code Protection
_FGS(GCP_OFF);  

EE_UINT8 myCounts = 0;     	

void initLeds (void)
{

	TRISDbits.TRISD0 = 0;
	TRISDbits.TRISD1 = 0;
	TRISDbits.TRISD2 = 0;
	TRISDbits.TRISD3 = 0;
	TRISDbits.TRISD4 = 0;
	TRISDbits.TRISD5 = 0;
	TRISDbits.TRISD6 = 0;
	TRISDbits.TRISD7 = 0;
	TRISDbits.TRISD8 = 0;
	TRISDbits.TRISD9 = 0;
	TRISDbits.TRISD10 = 0;
	TRISDbits.TRISD11 = 0;
	TRISDbits.TRISD12 = 0;
	TRISDbits.TRISD13 = 0;
	TRISDbits.TRISD14 = 0;
	TRISDbits.TRISD15 = 0;
}

void initPortB (void)
{

	TRISBbits.TRISB9 = 0;
	TRISBbits.TRISB10 = 0;
}

void SetLed ( int led, int on_off ){

	switch (led){
	case 0:
		LATDbits.LATD0 = on_off;
		break;
	case 1:
		LATDbits.LATD1 = on_off;
		break;
	case 2:
		LATDbits.LATD2 = on_off;
		break;
	case 3:
		LATDbits.LATD3 = on_off;
		break;
	}
}

void initButtons (void)
{
	TRISGbits.TRISG6 = 1;  		
	TRISGbits.TRISG7 = 1;  		
	TRISGbits.TRISG8 = 1;  		
	TRISGbits.TRISG9 = 1;  		

}
bool getButton ( int j ){

	switch (j){
		case 0:
			return !PORTGbits.RG6;
		case 1:
			return !PORTGbits.RG7;
		case 2:
			return !PORTGbits.RG8;
		case 3:
			return !PORTGbits.RG9;
	}
	return 0;
}
	
void initPanel( void ){
	EE_lcd_init();
	EE_lcd_clear();
	EE_lcd_home();
    EE_lcd_puts("Floor 0 Dir -");
	EE_lcd_line2();
    EE_lcd_puts("Doors C 0C1C2C3C");

}

void dummyDispatch(void)
{
//Mirror of PBL
	if (light0 == 1) {
		is0=1;
		Delay_Us ( Delay_20mS_Cnt );
	}
	if (b0==0 && light0==0 && lock0==0);
	if (lock0==1);

	if (light1 == 1) {
		is1=1;
		Delay_Us ( Delay_20mS_Cnt );
	}
	if (b1==0 && light1==0 && lock1==0);
	if (lock1==1);
	
	if (light2 == 1) {
		is2=1;
		Delay_Us ( Delay_20mS_Cnt );
	}
	if (b2==0 && light2==0 && lock2==0);
	if (lock2==1);
	if (light3 == 1) {
		is3=1;
		Delay_Us ( Delay_20mS_Cnt );
	}
	if (b3==0 && light3==0 && lock3==0);
	if (lock3==1);
// Mirror of Cabin
//	Delay_Us ( 5 * Delay_20mS_Cnt );

	if ( is0 || is1 || is2 || is3 ) {
		open = 1;
		close = 0;
	}

	if ((dummyState == 0)&&( is0 || is1 || is2 || is3 )) {
		dummyState=1;
		return;
	}
	if ((dummyState == 1) && !(lock0 && lock1 && lock2 && lock3)) {
		dummyState=0;
		open = 0;
		close = 1;
		is0 = is1 = is2 = is3 = 0;
		return;
	}
	if (dummyState == 2) {
		dummyState=0;
		return;
	}
}

void T1_program(void)
{
	T1CON = 0;		    /* Stops the Timer1 and reset control reg	*/
	TMR1  = 0;		    /* Clear contents of the timer register	*/
	PR1   = 0xc350;		/* Load the Period register wit the value 50000	ticks = 25 ms*/
	IPC0bits.T1IP = 5;	/* Set Timer1 priority to 1		*/
	IFS0bits.T1IF = 0;	/* Clear the Timer1 interrupt status flag	*/
	IEC0bits.T1IE = 1;	/* Enable Timer1 interrupts		*/
	T1CONbits.TON = 1;	/* Start Timer1 with prescaler settings at 1:1
				* and clock source set to the internal 
				* instruction cycle			*/
	T1CONbits.TCKPS1 = 0;	
	T1CONbits.TCKPS0 = 1;	
}
/* Clear the Timer1 interrupt status flag */
void T1_clear(void)
{
	IFS0bits.T1IF = 0;
}
void _ISR _T1Interrupt (void)
{
	/* clear the interrupt source */
	T1_clear();
//	setLed ( 0, on_off);
//	on_off = ! on_off;
	myCounts++;
}

int main(void)
{
	int k, b;
	/* Application Init */

	initButtons();
	initPortB();
	initPanel();
	initLeds();
	T1_program();

	initClk();

//FSM initialization
	PBLInit (&pbl0);
	PBLInit (&pbl1);
	PBLInit (&pbl2);
	PBLInit (&pbl3);
	PBLInitLocals (&pbl0, &lock0, &b0, &light0, &is0);
	PBLInitLocals (&pbl1, &lock1, &b1, &light1, &is1);
	PBLInitLocals (&pbl2, &lock2, &b2, &light2, &is2);
	PBLInitLocals (&pbl3, &lock3, &b3, &light3, &is3);

	CabinInit (&cbn);
	ControllerInit (&cnt);
	MotorInit (&mtr);

//main program
	while ( 1 ) {
		PBLDispatch(&pbl0);
		PBLDispatch(&pbl1);
		PBLDispatch(&pbl2);
		PBLDispatch(&pbl3);
//		Delay_Us ( 2 * Delay_20mS_Cnt );
		CabinDispatch (&cbn);
		ControllerDispatch (&cnt);
//		ControllerDispatch (&cnt);
		MotorDispatch (&mtr);

//		dummyDispatch();
		tickClk();
	}
	/* Forever loop: background activities (if any) should go here */
	for (;;);
	
	return 0;
}

void _ISR _CNInterrupt(void)
{
	/* la routine va in esecuzione ad ogni variazione del pin RB0.
	   In questo esempio il codice di gestione della sorgente viene
	   eseguito solo sul livelo basso del pin.
	   Nel caso del livello alto l'overhead  solo il test del pin */
	
	if(!PORTBbits.RB0)
	{
		// codice da eseguire
		

	}

	IFS1bits.CNIF = 0;
}