#include "motorshaft.h"
#include "delays.h"
#include "types.h"
#include "globals.h"

static myclock msclk;

static enum State {
	AT0, AT0DU,
	FROM0TO1, AT1, AT1DU, AT1DD, AT1STOP,
	FROM1TO2, AT2, AT2DU, AT2DD, AT2STOP,
	FROM2TO3, AT3, AT3DD
};
static enum Signal {
	ABSENT, GOUP, GODOWN, STAY, EXPDELAY, EXPDELAYUP, EXPDELAYDOWN
};

void MotorInit(Motor *me){
	me->state_ = AT0;
	is0 = 1; //cabin starts closed
	resetClk(&msclk);
}
void MotorTran_(Motor *me, unsigned int dest)
{
	me->state_ = dest;	
}
void MotorDisplay (Motor *me){

	EE_lcd_goto ( 6, 0 );
/*	EE_lcd_putc( '0'+me->state_ );
	return;
*/
	switch (me->state_) {
	  case AT0: 
		EE_lcd_putc( '0' );
		EE_lcd_goto ( 7, 0 );
		EE_lcd_putc(' ');
		break;
	  case FROM0TO1:
		EE_lcd_putc( 'O' );
		EE_lcd_goto ( 7, 0 );
		if (dir)		
			EE_lcd_putc( (unsigned char)126 );
		else
			EE_lcd_putc( (unsigned char)127 );
		break;
	  case AT1: 
		EE_lcd_putc( '1' );
		EE_lcd_goto ( 7, 0 );
		EE_lcd_putc(' ');
		break;
	  case FROM1TO2:
		EE_lcd_putc( '1' );
		EE_lcd_goto ( 7, 0 );
		if (dir)		
			EE_lcd_putc( (unsigned char)126 );
		else
			EE_lcd_putc( (unsigned char)127 );
		break;
	  case AT2: 
		EE_lcd_putc( '2' );
		EE_lcd_goto ( 7, 0 );
		EE_lcd_putc(' ');
		break;
	  case FROM2TO3:
		EE_lcd_putc( '2' );
		EE_lcd_goto ( 7, 0 );
		if (dir)		
			EE_lcd_putc( (unsigned char)126 );
		else
			EE_lcd_putc( (unsigned char)127 );
		break;
	  case AT3:
		EE_lcd_putc( '3' );
		EE_lcd_goto ( 7, 0 );
		EE_lcd_putc(' ');
		break;
	}
	EE_lcd_goto ( 12, 0 );
	if (dir)
		EE_lcd_putc( 'U' );
	else 
		EE_lcd_putc( 'D' );

}

enum Signal MotorGetSignals(Motor *me){
	if (halt)
		return STAY;
	if (up && ExpAlarm(&msclk))
		return EXPDELAYUP;
	if (down && ExpAlarm(&msclk))
		return EXPDELAYDOWN;
	if (ExpAlarm(&msclk))
		return EXPDELAY;
	if (up)
		return GOUP;
	if (down)
		return GODOWN;
	return ABSENT;
}
void MotorDispatch(Motor *me){

	enum Signal s = MotorGetSignals(me);
	switch ( me->state_ ){
		case AT0: 
			switch ( s ) {
				case GOUP:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT0DU );
					break;
				case STAY:
					ishalt = 1;
					break;
			}
			break;
		case AT0DU:
			switch(s) {
				case EXPDELAYUP:
					is0 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM0TO1 );
				break;
			}
			break;
		case FROM0TO1:
			switch ( s ) {
				case EXPDELAYUP:
					is1 = 1;
					SetAlarm(&msclk, 0, DELAYF);
					MotorTran_ ( me, AT1 );
					break;
				case EXPDELAYDOWN:
					is0 = 1;
					MotorTran_ ( me, AT0 );
					break;
			}
			break;
		case AT1: 
			switch ( s ) {
				case EXPDELAYUP:
					SetAlarm(&msclk, 0, DELAYIF);
					is1 = 0;
					MotorTran_ ( me, FROM1TO2 );
					break;
				case EXPDELAYDOWN:
					SetAlarm(&msclk, 0, DELAYIF);
					is1 = 0;
					MotorTran_ ( me, FROM0TO1 );
					break;
				case STAY:
					ishalt = 1;
					MotorTran_ ( me, AT1STOP );
					resetClk(&msclk);
					break;
			}
			break;
		case AT1STOP: 
			switch ( s ) {
				case GOUP:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT1DU );
					break;
				case GODOWN:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT1DD );
					break;
			}
			break;
		case AT1DU:
			switch(s) {
				case EXPDELAYUP:
					is1 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM1TO2 );
				break;
			}
			break;
		case AT1DD:
			switch(s) {
				case EXPDELAYDOWN:
					is1 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM0TO1 );
				break;
			}
			break;
		case FROM1TO2:
			switch ( s ) {
				case EXPDELAYUP:
					is2 = 1;
					SetAlarm(&msclk, 0, DELAYF);
					MotorTran_ ( me, AT2 );
					break;
				case EXPDELAYDOWN:
					is1 = 1;
					SetAlarm(&msclk, 0, DELAYF);
					MotorTran_ ( me, AT1 );
					break;
/*				case EXPDELAY:
					ishalt = 1;
					break;
*/
			}

			break;
		case AT2: 
			switch ( s ) {
				case EXPDELAYUP:
					SetAlarm(&msclk, 0, DELAYIF);
					is2 = 0;
					MotorTran_ ( me, FROM2TO3 );
					break;
				case EXPDELAYDOWN:
					SetAlarm(&msclk, 0, DELAYIF);
					is2 = 0;
					MotorTran_ ( me, FROM1TO2 );
					break;
				case STAY:
					ishalt = 1;
					MotorTran_ ( me, AT2STOP );
					resetClk(&msclk);
					break;
			}
			break;
		case AT2STOP: 
			switch ( s ) {
				case GOUP:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT2DU );
					break;
				case GODOWN:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT2DD );
					break;
			}
			break;
		case AT2DU:
			switch(s) {
				case EXPDELAYUP:
					is2 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM2TO3 );
				break;
			}
			break;
		case AT2DD:
			switch(s) {
				case EXPDELAYDOWN:
					is2 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM1TO2 );
				break;
			}
			break;
		case FROM2TO3:
			switch ( s ) {
				case EXPDELAYUP:
					is3 = 1;
					MotorTran_ ( me, AT3 );
					break;
				case EXPDELAYDOWN:
					is2 = 1;
					SetAlarm(&msclk, 0, DELAYF);
					MotorTran_ ( me, AT2 );
					break;
			}
			break;
		case AT3DD:
			switch(s) {
				case EXPDELAYDOWN:
					is3 = 0;
					SetAlarm(&msclk, 0, DELAYIF);
					MotorTran_ ( me, FROM2TO3 );
				break;
			}
			break;
		case AT3: 
			switch ( s ) {
 			case GODOWN:
					ishalt = 0;
					SetAlarm(&msclk, 0, DELAYHF);
					MotorTran_ ( me, AT3DD );
					break;
			case STAY:
					ishalt = 1;
					break;
			}
			break;
	}
	MotorDisplay( me );
}



					

