#include "pbl.h"
#include "types.h"
#include "globals.h"
#include "board.h"

static EE_UINT8 id = 0;
static enum State {
	IDLE, REQUESTED, ATFLOOROPEN
};
static enum Signal {
	ABSENT, BUTTON,	ISOPEN,	ATFLOOR_AND_OPEN
};

void PBLInit(PBL *me){
	me->state_ = IDLE;
	me->pblID_ = id;
	id++;
}

void PBLInitLocals(PBL *me, bool *lck, bool *bttn, bool *lght, bool *isat)
{
	me->lock_ = lck;
	me->button_ = bttn;
	me->light_ = lght;
	me->atfloor_ = isat;
	*lck = 1;
	*bttn = 0;
	*lght = 0;
}

enum Signal PBLGetSignals(PBL *me){
	switch ( me->state_ ){
		case IDLE:
			if (getButton(me->pblID_))
				return BUTTON;
		case REQUESTED:
			if (*(me->atfloor_) && isopen)
				return ATFLOOR_AND_OPEN;
		case ATFLOOROPEN:
			if (isopen == 0)
				return ISOPEN;
	}
}

void PBLSetLight_(PBL *me, int on_off )
{
	*(me->light_) = on_off;
	SetLed(me->pblID_, on_off);
}

void PBLTran_(PBL *me, unsigned int dest)
{
	me->state_ = dest;
}

void PBLDisplayLock_(PBL *me, bool lck)
{
	EE_lcd_goto ( 9+2*(me->pblID_), 1 );
	if (lck)
		EE_lcd_putc( 'C' );
	else 
		EE_lcd_putc( 'O' );
}

void PBLDispatch(PBL *me)
{
	enum Signal s = PBLGetSignals(me);
	switch ( me->state_ ){
		case IDLE:
			switch ( s ) {
				case BUTTON:
					*(me->button_) = 1;
					PBLSetLight_ (me, 1 );
					PBLTran_ (me, REQUESTED );
					break;
			}
			break;	
		case REQUESTED:
			switch ( s ) {
				case ATFLOOR_AND_OPEN:
					*(me->button_) = *(me->lock_) = 0;
					PBLSetLight_ (me, 0 );
					PBLDisplayLock_(me, 0);
					PBLTran_ (me, ATFLOOROPEN );
					break;
			}
			break;
		case ATFLOOROPEN:
			switch ( s ) {
				case ISOPEN:
					Delay_Us ( Delay_20mS_Cnt );
					*(me->lock_) = 1;
					PBLDisplayLock_(me, 1);
					PBLTran_ (me, 	IDLE );
					break;
			}
			break;
	}
}



					

